jQuery(function ($) {

  $.HatenaBookmark = function (userSettings) {

    this.settings = $.extend(true, {
      NGurl  : [],
      NGword : []
    }, userSettings);

    this.category = {
      names  : ['all', 'social', 'economics', 'life', 'knowledge', 'it', 'entertainment', 'game', 'fun'],
      labels : ['総合', '世の中', '政治と経済', '暮らし', '学び', 'テクノロジー', 'エンタメ', 'アニメとゲーム', 'おもしろ']
    };
    this.panelContainer = $('#container_tabpanel');
    this.data = {};

    this.init();
  };

  $.HatenaBookmark.prototype = {

    init : function () {
      var that = this;
      that.addShorcutKey();
      that.refresh();
    },

    switchTab : function (element) {
      var tab = $(element);
      var targetPanels;

      // 現在選択されているタブなら何もしない
      if (tab.attr('aria-selected') === 'true') {
        return false;
      }

      // 表示変更対対象となるタブパネルのID配列
      targetPanels = tab.attr('aria-controls').split(/\s+/);

      // 現在選択されている他のタブを非選択状態にし、自身を選択状態にする
      tab.siblings('[aria-selected="true"]').attr('aria-selected', 'false');
      tab.attr('aria-selected', 'true');

      // 表示変更対象のタブパネル操作 (配列なのでループ処理する)
      // ID検索して aria-hidden 属性の値 (Bool) を変更する
      // 対象ではないパネルは true, 対象となるものは false
      $.each(targetPanels, function (i, iv) {
        var panel = $(document.getElementById(iv));
        panel.siblings('[role="tabpanel"]').filter('[aria-hidden="false"]').attr('aria-hidden', 'true');
        panel.attr('aria-hidden', 'false');
      });
    },

    addShorcutKey : function () {

      var that = this;
      var tab = $('[role="tab"]');
      var categoryTab = $('#tablist_category').find('[role="tab"]');
      var typeTab = $('#tablist_type').find('[role="tab"]');

      tab.on('click keypress', function (e) {
        e.preventDefault();
        if (e.which === 13 || e.which === 1) {
          that.switchTab(this);
        }
      });

      $(window).on('keypress', function (e) {
        var key = e.which;

        // タブ変更
        // 48 = キーボードの "0", 57 が "9"
        if (key > 48 && key < 58) {
          that.switchTab(categoryTab.eq(key - 49));
        }
        if (key === 48) {
          that.switchTab(typeTab.filter('[aria-selected="false"]'));
        }
        // リフレッシュ
        if (key === 82 && e.shiftKey) {
          if (that.panelContainer.attr('aria-busy') === 'false') {
            that.refresh();
          }
        }
      });
    },

    getData : function () {
      var that = this;
      var dfd = new $.Deferred();

      $.each(that.category.names, function (i, iv) {

        that.data[iv] = {};

        $.each(['hotentry', 'entrylist'], function (j, jv) {
          var type = (jv === 'hotentry') ? 'hot' : 'new';
          var query = 'SELECT title, link, date, bookmarkcount FROM feed WHERE url="http://b.hatena.ne.jp/' + jv;
          if (iv !== 'all') {
            query += '/' + iv;
          }
          query += '.rss"';
          $.ajax({
            url      : "http://query.yahooapis.com/v1/public/yql",
            data     : {
              format : 'json',
              q      : query
            },
            dataType : 'jsonp'
          }).done(function (data) {
              that.data[iv][type] = {};
              that.data[iv][type]['list'] = that.formatData(data['query']['results']['item']);
              if (i + 1 === that.category.names.length && j === 1) {
                dfd.resolve();
              }
            });
        });
      });
      return dfd.promise();
    },

    formatData : function (itemlist) {

      var that = this;
      var newlist = [];

      $.each(itemlist, function (i, iv) {

        var skip = false;

        $.each(that.settings.NGurl, function (j, jv) {
          if (iv['link'].search(jv) > -1) {
            skip = true;
            return false;
          }
          return true;
        });

        if (!skip) {
          $.each(that.settings.NGword, function (j, jv) {
            if (iv['title'].search(jv) > -1) {
              skip = true;
              return false;
            }
            return true;
          });
        }

        if (!skip) {
          iv['datelabel'] = convertDate(iv['date']);
          newlist.push(iv);
        }
      });

      return newlist;

      // 日付を見やすくする
      function convertDate(str) {
        var date = new Date(str);
        return date.getFullYear() + '/' +
               zeroPadding(date.getMonth() + 1) + '/' +
               zeroPadding(date.getDate()) + ' ' +
               zeroPadding(date.getHours()) + ':' +
               zeroPadding(date.getMinutes());

        // 日時用ゼロパディング
        function zeroPadding(num) {
          return ('0' + num).slice(-2);
        }
      }
    },

    output : function () {
      var that = this;
      $.each(that.data, function (i, iv) {
        $.each(iv, function (j, jv) {
          var html = $.parseHTML(tmpl('tmpl_item', jv));
          $('#entrylist_' + i + '_' + j).empty().append(html);
        });
      });
    },

    refresh : function () {
      var that = this;
      that.panelContainer.attr('aria-busy', 'true');
      that.getData().done(function () {
        that.output();
        that.panelContainer.attr('aria-busy', 'false');
      });
    }
  };
});
